from unittest.mock import patch
import pytest
from typer.testing import CliRunner
from CTADIRAC.Core.Utilities.typer_callbacks import PRODUCTION_FIELDS
from CTADIRAC.ProductionSystem.scripts.cta_prod_get_all import (
    extract_user_name_from_dn,
    fill_columns,
    app,
)
from tests.unit.production import PRODUCTION_RESULTS

runner = CliRunner()


def test_extract_user_name_from_dn():
    name = "Surname Lastname"
    dn = f"/O=GRID-FR/C=FR/O=CNRS/OU=LUPM/CN={name}"
    result = extract_user_name_from_dn(dn)
    assert result == name


def test_fill_columns():
    result = fill_columns(PRODUCTION_RESULTS[0], PRODUCTION_FIELDS, True)
    assert isinstance(result, list)


@pytest.fixture
def mock_production_client():
    with patch(
        "DIRAC.ProductionSystem.Client.ProductionClient.ProductionClient"
    ) as mock_client:
        yield mock_client


def test_main(mock_production_client):
    mock_client_instance = mock_production_client.return_value
    mock_client_instance.getProductions.return_value = {
        "OK": True,
        "Value": PRODUCTION_RESULTS,
    }

    result = runner.invoke(app, ["--long", "--cond", '{"Status": "Active"}'])
    assert result.exit_code == 0
    mock_client_instance.getProductions.assert_called_once_with(
        condDict={"Status": "Active"}
    )
    assert PRODUCTION_RESULTS[0]["AuthorGroup"] in result.stdout



#!/usr/bin/env python

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

Script.setUsageMessage(
    """
Treat the incomplete transfers by removing the replicas at source SEs but not at the destination SE

Usage:
   cta-prod-treat-incomplete-transfers <ascii file with list of (transID lfn)> <destSE>

Arguments:
  <ascii file with list of (transID lfn)> : input file with (transID lfn) to treat
  <destSE>                                : destination SE at which the Replica must be kept
"""
)

Script.parseCommandLine(ignoreErrors=True)

from DIRAC.DataManagementSystem.Client.DataManager import DataManager


def read_inputs(file_path):
    content = open(file_path).readlines()
    resList = []
    for line in content:
        transID = line.strip().split(" ")[0]
        lfn = line.strip().split(" ")[1]
        if line != "\n":
            resList.append((transID, lfn))
    return resList


def removeSourceReplica(sourceSEs, lfn):
    dm = DataManager()
    for SE in sourceSEs:
        res = dm.removeReplica(SE, lfn)
        if not res["OK"]:
            gLogger.error("Error removing source replicas for:", lfn)
            return res["Message"]
    return DIRAC.S_OK()


@Script()
def main():
    args = Script.getPositionalArgs()

    if len(args) != 2:
        Script.showHelp()

    infile = args[0]
    destSE = args[1]
    resList = read_inputs(infile)
    tc = TransformationClient()

    sourceSEs = [
        "DESY-ZN-Disk",
        "LPNHE-Disk",
        "CNAF-Disk",
        "CYF-STORM-Disk",
        "LAPP-Disk",
        "CEA-Disk",
        "CC-IN2P3-Disk",
        "POLGRID-Disk",
        "LANCASTER-Disk",
        "CC-IN2P3-Tape",
        "CNAF-Tape",
        "DESY-ZN-Tape",
    ]

    if destSE in sourceSEs:
        sourceSEs.remove(destSE)

    # To do: improve by passing a list with all lfns to setFileStatusForTransformation
    for transID, lfn in resList:
        res = removeSourceReplica(sourceSEs, lfn)
        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(-1)
        res = tc.setFileStatusForTransformation(
            transID, newLFNsStatus="Processed", lfns=[lfn]
        )
        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(-1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Add a dataset from a query specified in a json file, e.g.:

{"MCCampaign": "PROD5b",
"particle": "proton",
"site": "Paranal"}

The created dataset will get the name of the json file (without the extension)
Use the --dry-run option to display the list of files corresponding to the query in the json file without creating the dataset

Usage:
   cta-prod-add-dataset <json file with dataset query dict>
   cta-prod-add-dataset <json file with dataset query dict> --dry-run
Arguments:
   json file with dataset query dict
   The file should have the .json extension, e.g. my_dataset.json
"""

__RCSID__ = "$Id$"

import json
import ast
import os

import DIRAC
from DIRAC.Core.Base.Script import Script
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


@Script()
def main():
    fc = FileCatalogClient()
    Script.registerSwitch(
        "", "dry-run", "shows the list of files corresponding to the dataset query"
    )
    switches, argss = Script.parseCommandLine(ignoreErrors=True)

    inputJsonList = []
    dry_run = False
    for switch in switches:
        if switch[0] == "dry-run":
            dry_run = True
    if len(argss) == 1:
        if not argss[0].endswith(".json"):
            gLogger.notice(
                "\nReading list of json files with dataset description from input file: %s\n"
                % argss[0]
            )
            inputJsonList = read_inputs_from_file(argss[0])
        else:
            inputJsonList.append(argss[0])
    else:
        Script.showHelp()

    f = open(argss[0] + ".datasets", "w")
    for inputJson in inputJsonList:
        if not inputJson.endswith(".json"):
            gLogger.error("Dataset description files must have .json extension")
            DIRAC.exit(-1)
        # read the meta data query from json file
        content = open(inputJson).readlines()
        for line in content:
            json_string = json.dumps(ast.literal_eval(line))

        MDdict = json.loads(json_string)

        # ensure the integer values are written in the right format
        for key, value in MDdict.items():
            if isinstance(value, float):
                MDdict[key] = {"=": value}

        datasetName = os.path.basename(inputJson).split(".json")[0]
        if dry_run:
            res = fc.findFilesByMetadata(MDdict, path="/")
            if not res["OK"]:
                gLogger.error("Can not access File Catalog:", res["Message"])
                DIRAC.exit(-1)
            lfnList = sorted(res["Value"])
            if len(lfnList) == 0:
                gLogger.notice("No files found for dataset:", datasetName)
            else:
                datasetLFN = datasetName + ".lfns"
                gLogger.notice("\ndry run:")
                gLogger.notice(f"{len(lfnList)} files found for dataset: {datasetName}")
                gLogger.notice("dumped in:", datasetLFN)
                f0 = open(datasetLFN, "w")
                for lfn in lfnList:
                    f0.write(f"{lfn}\n")
        else:
            datasetPath = os.path.join("/vo.cta.in2p3.fr/datasets/", datasetName)
            res = fc.addDataset({datasetPath: MDdict})

            if not res["OK"]:
                gLogger.error(f"Failed to add dataset {datasetName}: {res['Message']}")
                DIRAC.exit(-1)

            if datasetName in res["Value"]["Failed"]:
                gLogger.error(
                    "Failed to add dataset %s: %s"
                    % (datasetName, res["Value"]["Failed"][datasetName])
                )
                DIRAC.exit(-1)

            gLogger.notice("Successfully added dataset:", datasetName)
        f.write(datasetName + "\n")

    gLogger.notice("Datasets list dumped in ", argss[0] + ".datasets")
    DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()



#!/usr/bin/env python
from typing import NoReturn
import numpy as np
import typer
from rich.progress import Progress, SpinnerColumn, TextColumn
import matplotlib.pyplot as plt
from DIRAC.Core.Base.Script import Script
from DIRAC.WorkloadManagementSystem.Client.JobMonitoringClient import (
    JobMonitoringClient,
)
from DIRAC.TransformationSystem.Client.TransformationClient import (
    TransformationClient,
)
from DIRAC import gLogger

from CTADIRAC.Core.Utilities.typer_callbacks import (
    ALLOWED_PARAMETERS,
    PARAMETER_DICT,
    job_fields_arg_callback,
    job_parameters_callback,
    transformation_fields_arg_callback,
)

script = Script()
# Disabling DIRAC script argument parsing
script.localCfg.isParsed = True
script.parseCommandLine(ignoreErrors=True)

app = typer.Typer()

fields_arg_help = (
    "Fields to select jobs. VALUE can be a single value or a coma separated list"
)

default_job_parameter = "TotalCPUTime"
jobs_parameter: str = typer.Option(
    default_job_parameter,
    "--param",
    "-p",
    callback=job_parameters_callback,
    help="Job parameter to be selected. See 'show-parameter' command.",
    show_default=True,
)

do_save_plot: str = typer.Option(
    None, "--plot", help="Plot the data in file specified by filename"
)
do_save_data: str = typer.Option(
    None, "--data", help="Save the data in file specified by filename"
)
with_stat: str = typer.Option(None, "--stat", help="Add mean and std dev in plot")

trans_fields_arg = typer.Argument(
    ...,
    metavar="FIELD VALUE ...",
    callback=transformation_fields_arg_callback,
    help=fields_arg_help,
)
job_fields_arg = typer.Argument(
    ...,
    metavar="FIELD VALUE ...",
    callback=job_fields_arg_callback,
    help=fields_arg_help,
)

progress_description = "[progress.description]{task.description}"
progress_final_desc = "Getting jobs list [green]\N{check mark}"
progress_jobs_failed_desc = "Getting jobs list [red]\N{cross mark}"


def get_job_list_from_transformation(condition: dict) -> list:
    """Get the jobs ID list for given transformations status and type"""
    with Progress(
        SpinnerColumn(),
        TextColumn(progress_description),
        transient=False,
    ) as progress:
        prog_task = progress.add_task(description="Getting jobs list...", total=None)
        trans_client = TransformationClient()

        res = trans_client.getTransformations(condition)
        job_list = []
        if res["OK"]:
            for trans_info in res["Value"]:
                trans_id = trans_info["TransformationID"]
                tasks_info = trans_client.getTransformationTasks(
                    {"TransformationID": trans_id}
                )
                for task in tasks_info["Value"]:
                    job_list.append(int(task["ExternalID"]))
        else:
            gLogger.error(res)
        if job_list:
            progress.update(prog_task, description=progress_final_desc)
        else:
            progress.update(prog_task, description=progress_jobs_failed_desc)
    return job_list


def get_job_list_from_job(condition: dict) -> list:
    with Progress(
        SpinnerColumn(),
        TextColumn(progress_description),
        transient=False,
    ) as progress:
        task = progress.add_task(description="Getting jobs list...", total=None)
        jm_client = JobMonitoringClient()
        res_jobs = jm_client.getJobs(condition)
        if res_jobs["OK"]:
            if res_jobs["Value"]:
                progress.update(task, description=progress_final_desc)
                return res_jobs["Value"]
            else:
                progress.update(task, description=progress_jobs_failed_desc)
                return []
        else:
            progress.update(task, description=progress_jobs_failed_desc)
            gLogger.error(res_jobs)
            return []


def get_jobs_parameters(jobs_list: list):
    """Get the job parameters for given list"""
    with Progress(
        SpinnerColumn(),
        TextColumn(progress_description),
        transient=False,
    ) as progress:
        task = progress.add_task(description="Getting jobs parameters...", total=None)
        jm_client = JobMonitoringClient()
        result = jm_client.getJobParameters(jobs_list)
        if result["OK"]:
            progress.update(
                task, description="Getting jobs parameters [green]\N{check mark}"
            )
            return result["Value"]
        else:
            progress.update(
                task, description="Getting jobs parameters [red]\N{cross mark}"
            )
            raise typer.Exit(result)


def extract_parameter(
    jobs_list: list, parameter: str = PARAMETER_DICT[default_job_parameter]
) -> list:
    """Extract the parameter from the jobs parameters"""
    jobs_parameters = get_jobs_parameters(jobs_list)
    param_list = []
    for parameters in jobs_parameters.values():
        try:
            param_list.append(float(parameters.get(parameter)))
        except ValueError:
            param_list.append(parameters.get(parameter))
        except TypeError:
            continue
    if not param_list:
        raise typer.Exit("Empty parameter list")
    else:
        typer.echo(f"{len(param_list)} entries selected.")
    return param_list


def plot_data(
    param_list: list, file_name: str, jobs_parameter: str, with_stat: bool = False
) -> None:
    """Plot data"""
    with Progress(
        SpinnerColumn(),
        TextColumn(progress_description),
        transient=False,
    ) as progress:
        task = progress.add_task(description="Creating plot...", total=None)
        figure_name: str = f"{file_name}.png"
        plt.figure(figsize=(12, 8))
        plt.xlabel(jobs_parameter)
        plt.ylabel("Number of jobs")
        if isinstance(param_list[0], (int, float)):
            # Plot histogram for numerical data
            plt.hist(param_list, bins=10, color="steelblue", edgecolor="black")
            if with_stat:
                # Adding mean and std dev:
                mean = np.mean(param_list)
                std_dev = np.std(param_list)
                plt.legend(
                    [
                        f"Total jobs: {len(param_list)}, Mean: {mean:.2f}, Std Dev: {std_dev:.2f}"
                    ]
                )
        else:
            # Count the frequency of each unique string
            string_counts = {}
            for string in param_list:
                string_counts[string] = string_counts.get(string, 0) + 1

            # Plot bar chart for string data
            plt.bar(string_counts.keys(), string_counts.values(), color="steelblue")
            plt.xticks(rotation=45, ha="right")
            if with_stat:
                plt.legend([f"Total jobs: {len(param_list)}"])
        plt.tight_layout()
        plt.savefig(figure_name)
        progress.update(
            task, description=f"{figure_name} created [green]\N{check mark}"
        )


def save_data(param_list: list, file_name: str) -> None:
    """Print data list"""
    with Progress(
        SpinnerColumn(),
        TextColumn(progress_description),
        transient=False,
    ) as progress:
        task = progress.add_task(description="Saving data...", total=None)
        dat_file: str = f"{file_name}.dat"
        str_param_list: list[str] = [str(param) for param in param_list]
        with open(dat_file, "w") as f:
            f.write("\n".join(str_param_list))
        progress.update(task, description=f"{dat_file} created [green]\N{check mark}")


def finalize(
    jobs_list: list, jobs_parameter: str, plot: str, data: str, with_stat: bool
) -> None:
    typer.echo(f"{len(jobs_list)} jobs selected")
    param_list = extract_parameter(jobs_list, jobs_parameter)
    if plot:
        plot_data(param_list, plot, jobs_parameter, with_stat)
    if data:
        save_data(param_list, data)


# Command defintion:
@app.command("show-parameter")
def show_parameter() -> NoReturn:
    """Display the job parameters list"""
    typer.echo(ALLOWED_PARAMETERS)
    raise typer.Exit()


@app.command("transformation")
def transformation(
    plot: str = do_save_plot,
    with_stat: bool = with_stat,
    jobs_parameter: str = jobs_parameter,
    data: str = do_save_data,
    fields: list[str] = trans_fields_arg,
) -> None:
    """Select jobs parameters using transformation fields"""
    fields = fields[0]
    jobs_list = get_job_list_from_transformation(fields)
    if not jobs_list:
        raise typer.Exit("No jobs selected")
    finalize(jobs_list, jobs_parameter, plot, data, with_stat)


@app.command("job")
def job(
    plot: str = do_save_plot,
    with_stat: bool = with_stat,
    jobs_parameter: str = jobs_parameter,
    data: str = do_save_data,
    fields: list[str] = job_fields_arg,
) -> None:
    """Select jobs parameters using job fields"""
    fields = fields[0]
    jobs_list = get_job_list_from_job(fields)
    if not jobs_list:
        raise typer.Exit("No jobs selected")
    finalize(jobs_list, jobs_parameter, plot, data, with_stat)


if __name__ == "__main__":
    app()



# imports
import json

# from types import StringTypes
# Import sqlachemy modules to create objects mapped with tables
from sqlalchemy import Table, Column, ForeignKey
from sqlalchemy import Integer, String
from sqlalchemy.orm import sessionmaker, relationship
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.engine.reflection import Inspector
from sqlalchemy import (
    create_engine,
    func,
    Integer,
    String,
    DateTime,
    exc,
    BigInteger,
)

# Declare a declarative_base to map objets and tables
from sqlalchemy.ext.declarative import declarative_base

# from DIRAC
from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.ConfigurationSystem.Client.Utilities import getDBParameters


provBase = declarative_base()

################################################################################
# wasInformedBy association table (n-n relation)
wasInformedBy_association_table = Table(
    "wasInformedBy",
    provBase.metadata,
    Column("internal_key", BigInteger, primary_key=True, autoincrement=True),
    Column("informant", BigInteger, ForeignKey("activities.internal_key")),
    Column("informed", BigInteger, ForeignKey("activities.internal_key")),
)

################################################################################
# Define the Activity class mapped to the activities table


class Activity(provBase):
    __tablename__ = "activities"
    ordered_attribute_list = [
        "id",
        "name",
        "startTime",
        "endTime",
        "comment",
        "activityDescription_id",
    ]
    other_display_attributes = ["name", "comment"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    id = Column(String)
    name = Column(String)
    startTime = Column(String)
    endTime = Column(String)
    comment = Column(String)

    # n-1 relation with ActivityDescription
    activityDescription_key = Column(
        BigInteger, ForeignKey("activityDescriptions.internal_key")
    )
    activityDescription = relationship("ActivityDescription")
    # n-n relation
    wasInformedBy = relationship(
        "Activity",
        secondary=wasInformedBy_association_table,
        primaryjoin=internal_key == wasInformedBy_association_table.c.informed,
        secondaryjoin=internal_key == wasInformedBy_association_table.c.informant,
    )

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"Activity.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response

    def get_description_key(self):
        return self.activityDescription_Key


################################################################################
# wasDerivedFrom association table (n-n relation)
wasDerivedFrom_association_table = Table(
    "wasDerivedFrom",
    provBase.metadata,
    Column("internal_key", BigInteger, primary_key=True),
    Column("generatedEntity", BigInteger, ForeignKey("entities.internal_key")),
    Column("usedEntity", BigInteger, ForeignKey("entities.internal_key")),
)

################################################################################
# Define the Entity class mapped to the entities table


class Entity(provBase):
    __tablename__ = "entities"
    ordered_attribute_list = [
        "id",
        "classType",
        "name",
        "location",
        "generatedAtTime",
        "invalidatedAtTime",
        "comment",
        "entityDescription_id",
    ]
    other_display_attributes = [
        "id",
        "name",
        "location",
        "generatedAtTime",
        "invalidatedAtTime",
        "comment",
    ]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    id = Column(String)
    name = Column(String)
    location = Column(String)
    generatedAtTime = Column(DateTime)
    invalidatedAtTime = Column(DateTime)
    comment = Column(String)

    # Heritage
    classType = Column(String)
    __mapper_args__ = {"polymorphic_identity": "entity", "polymorphic_on": classType}

    # n-1 relation with EntityDescription
    entityDescription_key = Column(
        BigInteger, ForeignKey("entityDescriptions.internal_key")
    )
    entityDescription = relationship("EntityDescription")
    # n-n relation
    wasDerivedFrom = relationship(
        "Entity",
        secondary=wasDerivedFrom_association_table,
        primaryjoin=internal_key == wasDerivedFrom_association_table.c.usedEntity,
        secondaryjoin=internal_key
        == wasDerivedFrom_association_table.c.generatedEntity,
    )

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"Entity.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the collection class withy the hadMember: TO BE DONE!


################################################################################
# Define the ValueEntity class mapped to the valueEntities table
class ValueEntity(Entity):
    __tablename__ = "valueEntities"
    # ordered_attribute_list = Entity.ordered_attribute_list+['value']
    ordered_attribute_list = Entity.ordered_attribute_list
    other_display_attributes = ["value"]

    # Internal key
    internal_key = Column(
        BigInteger, ForeignKey("entities.internal_key"), primary_key=True
    )

    # Model attributes
    value = Column(String)

    # Heritage
    __mapper_args__ = {"polymorphic_identity": "value"}

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"ValueEntity.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


####################################################################################################
# Define the DatasetEntity class mapped to the datasetEntities table


class DatasetEntity(Entity):
    __tablename__ = "datasetEntities"
    ordered_attribute_list = Entity.ordered_attribute_list
    other_display_attributes = ["ctadirac_guid"]

    # Internal key
    internal_key = Column(
        BigInteger, ForeignKey("entities.internal_key"), primary_key=True
    )

    # Model attributes
    # CTADIRAC attributes
    ctadirac_guid = Column(String)

    # Heritage
    __mapper_args__ = {"polymorphic_identity": "dataset"}

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"DatasetEntity.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the Used class mapped to the used table


class Used(provBase):
    __tablename__ = "used"
    ordered_attribute_list = [
        "role",
        "time",
        "activity_key",
        "entity_key",
        "usageDescription_key",
    ]
    other_display_attributes = ["role", "time"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    role = Column(String, nullable=True)
    time = Column(String)

    # n-1 relation with Activity
    activity_key = Column(BigInteger, ForeignKey("activities.internal_key"))
    activity = relationship("Activity", backref="used")
    # n-1 relation with Entity
    entity_key = Column(BigInteger, ForeignKey("entities.internal_key"))
    entity = relationship("Entity", backref="used")
    # n-1 relation with UsageDescription
    usageDescription_key = Column(
        BigInteger, ForeignKey("usageDescriptions.internal_key")
    )
    usageDescription = relationship("UsageDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"Used.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the WasGeneratedBy class mapped to the wasGeneratedBy table
class WasGeneratedBy(provBase):
    __tablename__ = "wasGeneratedBy"
    ordered_attribute_list = ["role", "activity_key", "entity_key"]
    other_display_attributes = ["role"]

    # Internal Key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    role = Column(String, nullable=True)

    # n-1 relation with Activity
    activity_key = Column(BigInteger, ForeignKey("activities.internal_key"))
    activity = relationship("Activity")
    # 0..1-1 relation with Entity
    entity_key = Column(BigInteger, ForeignKey("entities.internal_key"))
    entity = relationship("Entity", uselist=False)
    # n-1 relation with GenerationDescription
    generationDescription_key = Column(
        BigInteger, ForeignKey("generationDescriptions.internal_key")
    )
    generationDescription = relationship("GenerationDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"WasGeneratedBy.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the Agent class mapped to the agents table


class Agent(provBase):
    __tablename__ = "agents"
    ordered_attribute_list = [
        "id",
        "name",
        "type",
        "email",
        "comment",
        "affiliation",
        "phone",
        "address",
        "url",
    ]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    id = Column(String)
    name = Column(String)
    type = Column(String)
    comment = Column(String)
    email = Column(String)
    affiliation = Column(String)
    phone = Column(String)
    address = Column(String)
    url = Column(String)

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"Agent.{attribute}={self.__dict__[attribute]}\n"
        return response


################################################################################
# Define the WasAssociatedWith class mapped to the wasAssociatedWith table


class WasAssociatedWith(provBase):
    __tablename__ = "wasAssociatedWith"
    ordered_attribute_list = ["activity_key", "agent_key", "role"]
    other_display_attributes = ["role"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    role = Column(String, nullable=True)

    # n-1 relation with Activity
    activity_key = Column(BigInteger, ForeignKey("activities.internal_key"))
    activity = relationship("Activity", backref="wasAssociatedWith")
    # n-1 relation with Agent
    agent_key = Column(BigInteger, ForeignKey("agents.internal_key"))
    agent = relationship("Agent", backref="wasAssociatedWith")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"WasAssociatedWith.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the WasAttributedTo class mapped to the wasAttributedTo table


class WasAttributedTo(provBase):
    __tablename__ = "wasAttributedTo"
    ordered_attribute_list = ["entity_key", "agent_key", "role"]
    other_display_attributes = ["role"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    role = Column(String, nullable=True)

    # n-1 relation with Entity
    entity_key = Column(BigInteger, ForeignKey("entities.internal_key"))
    entity = relationship("Entity", backref="wasAttributedTo")
    # n-1 relation with Agent
    agent_key = Column(BigInteger, ForeignKey("agents.internal_key"))
    agent = relationship("Agent", backref="wasAttributedTo")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"WasAttributedTo.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the ActivityDescription class mapped to the activityDescriptions table


class ActivityDescription(provBase):
    __tablename__ = "activityDescriptions"
    ordered_attribute_list = [
        "name",
        "version",
        "description",
        "type",
        "subtype",
        "doculink",
    ]
    other_display_attributes = [
        "name",
        "version",
        "description",
        "type",
        "subtype",
        "doculink",
    ]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    name = Column(String)
    version = Column(String)
    description = Column(String)
    type = Column(String)
    subtype = Column(String)
    doculink = Column(String)

    # 1-n relation with ConfigFileDescription
    configFiles = relationship("ConfigFileDescription")
    # 1-n relation with ParameterDescription
    parameters = relationship("ParameterDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"ActivityDescription.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the EntityDescription class mapped to the entityDescriptions table


class EntityDescription(provBase):
    __tablename__ = "entityDescriptions"
    ordered_attribute_list = ["name", "type", "description", "doculink", "classType"]
    other_display_attributes = ["name", "type", "description", "doculink", "classType"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    name = Column(String)
    type = Column(String)
    description = Column(String)
    doculink = Column(String)
    classType = Column(String)

    # Heritage
    __mapper_args__ = {
        "polymorphic_identity": "entityDescription",
        "polymorphic_on": classType,
    }

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"EntityDescription.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the ValueDescription class mapped to the valueDescriptions table


class ValueDescription(EntityDescription):
    __tablename__ = "valueDescriptions"
    # ordered_attribute_list = EntityDescription.ordered_attribute_list + ['valueType','unit','ucd','utype']
    ordered_attribute_list = EntityDescription.ordered_attribute_list
    other_display_attributes = []

    # Internal key
    internal_key = Column(
        BigInteger, ForeignKey("entityDescriptions.internal_key"), primary_key=True
    )

    # Model attributes
    valueType = Column(String)
    unit = Column(String)
    ucd = Column(String)
    utype = Column(String)

    # Heritage
    __mapper_args__ = {"polymorphic_identity": "valueDescription"}

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"ValueDescription.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the DatasetDescription class mapped to the datasetDescriptions table


class DatasetDescription(EntityDescription):
    __tablename__ = "datasetDescriptions"
    # ordered_attribute_list = EntityDescription.ordered_attribute_list + ['contentType']
    ordered_attribute_list = EntityDescription.ordered_attribute_list
    other_display_attributes = []

    # Internal key
    internal_key = Column(
        BigInteger, ForeignKey("entityDescriptions.internal_key"), primary_key=True
    )

    # Model attributes
    contentType = Column(String)

    # Heritage
    __mapper_args__ = {"polymorphic_identity": "datasetDescription"}

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"DatasetDescription.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the UsageDescription class mapped to the usageDescriptions table


class UsageDescription(provBase):
    __tablename__ = "usageDescriptions"
    ordered_attribute_list = [
        "role",
        "description",
        "type",
        "multiplicity",
        "activityDescription_key",
        "entityDescription_key",
    ]
    # Key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    role = Column(String, nullable=True)
    description = Column(String)
    type = Column(String)
    multiplicity = Column(Integer)

    # n-1 relation with ActivityDescription
    activityDescription_key = Column(
        BigInteger, ForeignKey("activityDescriptions.internal_key")
    )
    activityDescription = relationship("ActivityDescription")
    # n-1 relation with EntityDescription
    entityDescription_key = Column(
        BigInteger, ForeignKey("entityDescriptions.internal_key")
    )
    entityDescription = relationship("EntityDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"UsageDescription.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


####################################################################################################
# Define the GenerationDescription class mapped to the generationDescriptions table


class GenerationDescription(provBase):
    __tablename__ = "generationDescriptions"
    ordered_attribute_list = [
        "role",
        "description",
        "type",
        "activityDescription_key",
        "entityDescription_key",
    ]
    other_display_attributes = ["role", "description", "type", "multiplicity"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    role = Column(String, nullable=True)
    description = Column(String)
    type = Column(String)
    multiplicity = Column(Integer)

    # Relations
    activityDescription_key = Column(
        BigInteger, ForeignKey("activityDescriptions.internal_key")
    )
    activityDescription = relationship("ActivityDescription")
    # n-1 relation with EntityDescription
    entityDescription_key = Column(
        BigInteger, ForeignKey("entityDescriptions.internal_key")
    )
    entityDescription = relationship("EntityDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += "GenerationDescription.{}={}\n".format(
                attribute,
                self.__dict__[attribute],
            )
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the WasConfiguredBy class mapped to the wasConfiguredBy table


class WasConfiguredBy(provBase):
    __tablename__ = "wasConfiguredBy"
    ordered_attribute_list = ["id", "artefactType"]
    other_display_attributes = ["artefactType"]

    # Internal keya
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    artefactType = Column(String, nullable=True)

    # n-1 relation with Activity
    activity_key = Column(BigInteger, ForeignKey("activities.internal_key"))
    activity = relationship("Activity", backref="WasConfiguredBy", uselist=False)
    # 1-1 relation with ConfigFile
    configFile_key = Column(BigInteger, ForeignKey("configFiles.internal_key"))
    configFile = relationship("ConfigFile", uselist=False)
    # 1-1 relation with Parameter
    parameter_key = Column(BigInteger, ForeignKey("parameters.internal_key"))
    parameter = relationship("Parameter", uselist=False)

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"WasConfiguredBy.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the Parameter class mapped to the parameters table
class Parameter(provBase):
    __tablename__ = "parameters"
    ordered_attribute_list = ["name", "value"]
    other_display_attributes = ["name", "value"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    name = Column(String)
    value = Column(String)

    # n-1 relation with ParameterDescription
    parameterDescription_key = Column(
        BigInteger, ForeignKey("parameterDescriptions.internal_key")
    )
    parameterDescription = relationship("ParameterDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"Parameter.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the ConfigFile class mapped to the configFiles table
class ConfigFile(provBase):
    __tablename__ = "configFiles"
    ordered_attribute_list = ["name", "location", "comment"]
    other_display_attributes = ["name", "location", "comment"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    name = Column(String)
    location = Column(String)
    comment = Column(String)

    # n-1 relation with ConfigFileDescription
    configFileDescription_key = Column(
        BigInteger, ForeignKey("configFileDescriptions.internal_key")
    )
    configFileDescription = relationship("ConfigFileDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"ConfigFile.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the ParameterDescription class mapped to the parameters table
class ParameterDescription(provBase):
    __tablename__ = "parameterDescriptions"
    ordered_attribute_list = [
        "name",
        "valueType",
        "description",
        "unit",
        "ucd",
        "utype",
        "min",
        "max",
        "default",
        "options",
        "activityDescription_key",
    ]
    other_display_attributes = [
        "name",
        "valueType",
        "description",
        "unit",
        "ucd",
        "utype",
        "min",
        "max",
        "default",
        "options",
    ]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    name = Column(String)
    valueType = Column(String)
    description = Column(String)
    unit = Column(String)
    ucd = Column(String)
    utype = Column(String)
    min = Column(String)
    max = Column(String)
    default = Column(String)
    options = Column(String)

    # n-1 relation with ActivityDescription
    activityDescription_key = Column(
        BigInteger, ForeignKey("activityDescriptions.internal_key")
    )
    activityDescription = relationship("ActivityDescription")

    # Print method
    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += f"ParameterDescription.{attribute}={self.__dict__[attribute]}\n"
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
# Define the ConfigFileDescription class mapped to the configFileDescriptions table
class ConfigFileDescription(provBase):
    __tablename__ = "configFileDescriptions"
    ordered_attribute_list = [
        "name",
        "contentType",
        "description",
        "activityDescription_key",
    ]
    other_display_attributes = ["name", "contentType", "description"]

    # Internal key
    internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

    # Model attributes
    name = Column(String)
    contentType = Column(String)
    description = Column(String)

    # n-1 relation with ActivityDescription
    activityDescription_key = Column(
        BigInteger, ForeignKey("activityDescriptions.internal_key")
    )
    activityDescription = relationship("ActivityDescription")

    def __repr__(self):
        response = ""
        for attribute in self.ordered_attribute_list:
            response += "ConfigFileDescription.{}={}\n".format(
                attribute,
                self.__dict__[attribute],
            )
        return response

    # Other methods
    def get_display_attributes(self):
        response = {}
        for attribute in self.other_display_attributes:
            response["voprov:" + attribute] = self.__dict__[attribute]
        return response


################################################################################
class ProvenanceDB:
    """
    Class that defines the interactions with the tables of the ProvenanceDB.
    """

    def __getDBConnectionInfo(self, fullname):
        """Collect from the CS all the info needed to connect to the DB.
        This should be in a base class eventually
        """

        result = getDBParameters(fullname)
        if not result["OK"]:
            raise Exception(f"Cannot get database parameters: {result['Message']}")

        dbParameters = result["Value"]
        self.dbHost = dbParameters["Host"]
        self.dbPort = dbParameters["Port"]
        self.dbUser = dbParameters["User"]
        self.dbPass = dbParameters["Password"]
        self.dbName = dbParameters["DBName"]

    def __init__(self):
        """c'tor
        :param self: self reference
        """

        self.log = gLogger.getSubLogger("ProvenanceDB")
        # Initialize the connection info
        self.__getDBConnectionInfo("DataManagement/ProvenanceDB")

        runDebug = gLogger.getLevel() == "DEBUG"
        self.engine = create_engine(
            "postgresql://%s:%s@%s:%s/%s"
            % (self.dbUser, self.dbPass, self.dbHost, self.dbPort, self.dbName),
            echo=runDebug,
        )

        self.sessionMaker_o = sessionmaker(bind=self.engine)
        self.inspector = Inspector.from_engine(self.engine)

        # These are the list of tables that will be created.
        self.__initializeDB()

    def __initializeDB(self):
        """
        Create the tables, if they are not there yet
        """

        # sqlalchemy creates the database for me
        provBase.metadata.create_all(self.engine)

    def _sessionAdd(self, provInstance):
        session = self.sessionMaker_o()
        try:
            session.add(provInstance)
            session.commit()
            res = session.query(
                func.max(provInstance.internal_key).label("internal_key")
            ).one()
            session.commit()
            return S_OK({"internal_key": res.internal_key})
        except exc.IntegrityError as err:
            self.log.warn(f"insert: trying to insert a duplicate key? {err}")
            session.rollback()
            return S_ERROR("Key already exists")
        except exc.SQLAlchemyError as e:
            session.rollback()
            self.log.exception("insert: unexpected exception", lException=e)
            return S_ERROR(f"insert: unexpected exception {e}")
        finally:
            session.close()

    def _dictToObject(self, table, fromDict):
        """
        Add Agent
        :param agentDict:
        :return:
        """

        fromDict = (
            fromDict
            if isinstance(fromDict, dict)
            else json.loads(fromDict)
            if isinstance(fromDict, str)
            else {}
        )

        for key, value in fromDict.items():
            # The JSON module forces the use of UTF-8, which is not properly
            # taken into account in DIRAC.
            # Not shure we also need to verify StringTypes'
            if isinstance(value, str):
                value = value.encode()

            if value:
                setattr(table, key, value)

        return table

    def addAgent(self, rowDict):
        """
        Add Agent
        :param rowDict:
        :return:
        """

        agent = Agent()
        row = self._dictToObject(agent, rowDict)

        try:
            res = self.getAgentKey(agent.id)["Value"]["internal_key"]
            return S_OK({"internal_key": res})
        except BaseException:
            return self._sessionAdd(row)

    def addActivity(self, rowDict):
        """
        Add Activity
        :param rowDict:
        :return:
        """

        activity = Activity()
        row = self._dictToObject(activity, rowDict)

        return self._sessionAdd(row)

    def addWasAssociatedWith(self, rowDict):
        """
        Add WasAssociatedWith
        :param rowDict:
        :return:
        """

        wasAssociatedWith = WasAssociatedWith()
        row = self._dictToObject(wasAssociatedWith, rowDict)

        return self._sessionAdd(row)

    def addActivityDescription(self, rowDict):
        """
        Add ActivityDescription
        :param rowDict:
        :return:
        """

        activityDesc = ActivityDescription()
        row = self._dictToObject(activityDesc, rowDict)
        try:
            res = self.getActivityDescriptionKey(
                activityDesc.name, activityDesc.version
            )["Value"]["internal_key"]
            return S_OK({"internal_key": res})
        except BaseException:
            return self._sessionAdd(row)

    def addDatasetDescription(self, rowDict):
        """
        Add DatasetDescription
        :param rowDict:
        :return:
        """

        datasetDesc = DatasetDescription()
        row = self._dictToObject(datasetDesc, rowDict)
        try:
            res = self.getEntityDescriptionKey(datasetDesc.name)["Value"][
                "internal_key"
            ]
            return S_OK({"internal_key": res})
        except BaseException:
            return self._sessionAdd(row)

    def addUsageDescription(self, rowDict):
        """
        Add UsageDescription
        :param rowDict:
        :return:
        """

        usageDesc = UsageDescription()
        row = self._dictToObject(usageDesc, rowDict)
        try:
            res = self.getUsageDescriptionKey(
                usageDesc.activityDescription_key,
                usageDesc.entityDescription_key,
                usageDesc.role,
            )["Value"]["internal_key"]
            return S_OK({"internal_key": res})
        except BaseException:
            return self._sessionAdd(row)

    def addGenerationDescription(self, rowDict):
        """
        Add UsageDescription
        :param rowDict:
        :return:
        """

        generationDesc = GenerationDescription()
        row = self._dictToObject(generationDesc, rowDict)
        try:
            res = self.getGenerationDescriptionKey(
                generationDesc.activityDescription_key,
                generationDesc.entityDescription_key,
                generationDesc.role,
            )["Value"]["internal_key"]
            return S_OK({"internal_key": res})
        except BaseException:
            return self._sessionAdd(row)

    def addDatasetEntity(self, rowDict):
        """
        Add DatasetEntity
        :param rowDict:
        :return:
        """

        datasetEntity = DatasetEntity()
        row = self._dictToObject(datasetEntity, rowDict)
        return self._sessionAdd(row)

    def addWasAttributedTo(self, rowDict):
        """
        Add WasAttributedTo
        :param rowDict:
        :return:
        """

        wasAttributedTo = WasAttributedTo()
        row = self._dictToObject(wasAttributedTo, rowDict)
        return self._sessionAdd(row)

    def addUsed(self, rowDict):
        """
        Add Used
        :param rowDict:
        :return:
        """

        used = Used()
        row = self._dictToObject(used, rowDict)
        return self._sessionAdd(row)

    def addWasGeneratedBy(self, rowDict):
        """
        Add WasGeneratedBy
        :param rowDict:
        :return:
        """

        wasGeneratedBy = WasGeneratedBy()
        row = self._dictToObject(wasGeneratedBy, rowDict)
        return self._sessionAdd(row)

    def addValueEntity(self, rowDict):
        """
        Add ValueEntity
        :param rowDict:
        :return:
        """

        valueEntity = ValueEntity()
        row = self._dictToObject(valueEntity, rowDict)
        return self._sessionAdd(row)

    def addValueDescription(self, rowDict):
        """
        Add ValueDescription
        :param rowDict:
        :return:
        """

        valueDesc = ValueDescription()
        row = self._dictToObject(valueDesc, rowDict)
        try:
            res = self.getEntityDescriptionKey(valueDesc.name)["Value"]["internal_key"]
            return S_OK({"internal_key": res})
        except BaseException:
            return self._sessionAdd(row)

    def addWasConfiguredBy(self, rowDict):
        """
        Add WasConfiguredBy
        :param rowDict:
        :return:
        """

        wasConfiguredBy = WasConfiguredBy()
        row = self._dictToObject(wasConfiguredBy, rowDict)
        return self._sessionAdd(row)

    def addParameter(self, rowDict):
        """
        Add Parameter
        :param rowDict:
        :return:
        """

        parameter = Parameter()
        row = self._dictToObject(parameter, rowDict)
        return self._sessionAdd(row)

    def addConfigFile(self, rowDict):
        """
        Add ConfigFile
        :param rowDict:
        :return:
        """

        configFile = ConfigFile()
        row = self._dictToObject(configFile, rowDict)
        return self._sessionAdd(row)

    def addParameterDescription(self, rowDict):
        """
        Add ParameterDescription
        :param rowDict:
        :return:
        """

        parameterDescription = ParameterDescription()
        row = self._dictToObject(parameterDescription, rowDict)
        return self._sessionAdd(row)

    def addConfigFileDescription(self, rowDict):
        """
        Add ConfigFileDescription
        :param rowDict:
        :return:
        """

        configFileDescription = ConfigFileDescription()
        row = self._dictToObject(configFileDescription, rowDict)
        return self._sessionAdd(row)

    def getAgents(self):
        """
        Get Agents
        :return:
        """

        session = self.sessionMaker_o()
        agentIDs = []
        try:
            for instance in session.query(Agent):
                agentIDs.append(instance.internal_key)
            session.commit()
            return S_OK(agentIDs)
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getDatasetEntity(self, guid):
        """
        Get DatasetEntity
        :param guid
        :return:
        """

        session = self.sessionMaker_o()
        try:
            datasetEntity = (
                session.query(DatasetEntity)
                .filter(DatasetEntity.id == guid)
                .filter(DatasetEntity.invalidatedAtTime is None)
                .one()
            )
            return S_OK(datasetEntity.internal_key)
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def updateDatasetEntity(self, internal_key, invalidatedAtTime):
        """
        Update DatasetEntity
        :param internal_key, invalidatedAtTime
        :return:
        """

        session = self.sessionMaker_o()
        try:
            datasetEntity = (
                session.update(DatasetEntity)
                .where(DatasetEntity.internal_key == internal_key)
                .values(invalidatedAtTime=invalidatedAtTime)
            )
            session.commit()
            return S_OK()
        except e:
            return S_OK()
        finally:
            session.close()

    def getUsageDescription(self, activityDescription_key, role):
        """
        Get UsageDescription
        :param activityDescription_key, role
        :return: usageDescription.internal_key
        """

        session = self.sessionMaker_o()
        try:
            usageDescription = (
                session.query(UsageDescription)
                .filter(
                    UsageDescription.activityDescription_key == activityDescription_key,
                    UsageDescription.role == role,
                )
                .one()
            )
            session.commit()
            return S_OK(
                {
                    "internal_key": usageDescription.internal_key,
                    "entityDescription_key": usageDescription.entityDescription_key,
                }
            )
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getGenerationDescription(self, activityDescription_key, role):
        """
        Get GenerationDescription
        :param activityDescription_key, role
        :return: generationDescription.internal_key
        """

        session = self.sessionMaker_o()
        try:
            generationDescription = (
                session.query(GenerationDescription)
                .filter(
                    GenerationDescription.activityDescription_key
                    == activityDescription_key
                )
                .filter(GenerationDescription.role == role)
                .one()
            )
            session.commit()
            return S_OK(
                {
                    "internal_key": generationDescription.internal_key,
                    "entityDescription_key": generationDescription.entityDescription_key,
                }
            )
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getParameterDescription(self, activityDescription_key, parameter_name):
        """
        Get getParameterDescription
        :param activityDescription_key, parameter_name
        :return: parameterDescription.internal_key
        """

        session = self.sessionMaker_o()
        try:
            parameterDescription = (
                session.query(ParameterDescription)
                .filter(
                    ParameterDescription.activityDescription_key
                    == activityDescription_key,
                    ParameterDescription.name == parameter_name,
                )
                .one()
            )
            session.commit()
            return S_OK({"internal_key": parameterDescription.internal_key})
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getConfigFileDescription(self, activityDescription_key, configFile_name):
        """
        Get getConfigFileDescription
        :param activityDescription_key, configFile_name
        :return: configFileDescription.internal_key
        """

        session = self.sessionMaker_o()
        try:
            configFileDescription = (
                session.query(ConfigFileDescription)
                .filter(
                    ConfigFileDescription.activityDescription_key
                    == activityDescription_key,
                    ConfigFileDescription.name == configFile_name,
                )
                .one()
            )
            session.commit()
            return S_OK({"internal_key": configFileDescription.s})
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getActivityDescriptionKey(
        self, activityDescription_name, activityDescription_version
    ):
        """
        Get getActivityDescriptionKey
        :param activityDescription_name, activityDescription_version
        :return: ActivityDescription.internal_key
        """
        session = self.sessionMaker_o()
        try:
            activityDescription_list = (
                session.query(ActivityDescription)
                .filter(ActivityDescription.name == activityDescription_name)
                .filter(ActivityDescription.version == activityDescription_version)
                .all()
            )
            activityDescription_last = activityDescription_list[-1]
            return S_OK({"internal_key": activityDescription_last.internal_key})

        except NoResultFound:
            return S_OK()
        except BaseException:
            return S_OK()
        finally:
            session.close()

    def getEntityDescriptionKey(self, entityDescription_name):
        """
        Get getEntityDescriptionKey
        :param entityDescription_name
        :return: EntityDescription.internal_key
        """
        session = self.sessionMaker_o()
        try:
            entityDescription_list = (
                session.query(EntityDescription)
                .filter(EntityDescription.name == entityDescription_name)
                .all()
            )

            entityDescription_last = entityDescription_list[-1]
            return S_OK({"internal_key": entityDescription_last.internal_key})
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getUsageDescriptionKey(
        self, usageDescription_activity, usageDescription_entity, usageDescription_role
    ):
        """
        Get getUsageDescriptionKey
        :param usageDescription_activity, usageDescription_entity, usageDescription_role
        :return: UsageDescription.internal_key
        """
        session = self.sessionMaker_o()
        try:
            usageDescription_list = (
                session.query(UsageDescription)
                .filter(
                    UsageDescription.activityDescription_key
                    == usageDescription_activity
                )
                .filter(
                    UsageDescription.entityDescription_key == usageDescription_entity
                )
                .filter(UsageDescription.role == usageDescription_role)
                .all()
            )

            usageDescription_last = usageDescription_list[-1]
            return S_OK({"internal_key": usageDescription_last.internal_key})
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getGenerationDescriptionKey(
        self,
        generationDescription_activity,
        generationDescription_entity,
        generationDescription_role,
    ):
        """
        Get getGenerationDescriptionKey
        :param generationDescription_activity, generationDescription_entity, generationDescription_role
        :return: GenerationDescription.internal_key
        """
        session = self.sessionMaker_o()
        try:
            generationDescription_list = (
                session.query(GenerationDescription)
                .filter(
                    GenerationDescription.activityDescription_key
                    == generationDescription_activity
                )
                .filter(
                    GenerationDescription.entityDescription_key
                    == generationDescription_entity
                )
                .filter(GenerationDescription.role == generationDescription_role)
                .all()
            )

            generationDescription_last = generationDescription_list[-1]
            return S_OK({"internal_key": generationDescription_last.internal_key})
        except NoResultFound:
            return S_OK()
        finally:
            session.close()

    def getAgentKey(self, agent_id):
        """
        Get AgentKey
        :param agent_id
        :return: Agent.internal_key
        """
        session = self.sessionMaker_o()
        try:
            agent_list = session.query(Agent).filter(Agent.id == agent_id).all()
            agent_last = agent_list[-1]
            return S_OK({"internal_key": agent_last.internal_key})
        except NoResultFound:
            return S_OK()
        finally:
            session.close()



from copy import deepcopy

from CTADIRAC.Interfaces.API.CTAJob import MetadataDict
from CTADIRAC.Interfaces.API.CtapipeApplyModelsJob import CtapipeApplyModelsJob
from CTADIRAC.Interfaces.API.CtapipeMergeJob import CtapipeMergeJob
from CTADIRAC.Interfaces.API.CtapipeProcessJob import CtapipeProcessJob
from CTADIRAC.Interfaces.API.CtapipeTrainClassifierJob import CtapipeTrainClassifierJob
from CTADIRAC.Interfaces.API.CtapipeTrainEnergyJob import CtapipeTrainEnergyJob
from CTADIRAC.Interfaces.API.MCPipeJob import MCPipeJob
from CTADIRAC.Interfaces.API.MCSimTelProcessJob import MCSimTelProcessJob
from CTADIRAC.ProductionSystem.Client.WorkflowElement import (
    WorkflowElement,
    WorkflowElementDefinition,
)
from DIRAC.Interfaces.API.Job import Job
from tests.unit.production import (
    COMMON_CONFIG,
    CTAPIPE_PROCESS_OUTPUT_METADATA,
    MERGING_CONFIG_1,
    PROCESSING_CONFIG,
    SIM_TELARRAY_PROCESSING_CONFIG,
    SIMULATION_CONFIG,
    SIMULATION_OUTPUT_METADATA,
    WORKFLOW_CONFIG,
)


def test_workflow_element_definition() -> None:
    workflow_elmt = WorkflowElementDefinition("mcsimulation")
    assert isinstance(workflow_elmt.job, MCPipeJob)

    workflow_elmt = WorkflowElementDefinition("simtelprocessing")
    assert isinstance(workflow_elmt.job, MCSimTelProcessJob)

    workflow_elmt = WorkflowElementDefinition("ctapipeprocessing")
    assert isinstance(workflow_elmt.job, CtapipeProcessJob)

    workflow_elmt = WorkflowElementDefinition("merging")
    assert isinstance(workflow_elmt.job, CtapipeMergeJob)

    workflow_elmt = WorkflowElementDefinition("ctapipetrainenergy")
    assert isinstance(workflow_elmt.job, CtapipeTrainEnergyJob)

    workflow_elmt = WorkflowElementDefinition("ctapipetrainclassifier")
    assert isinstance(workflow_elmt.job, CtapipeTrainClassifierJob)

    workflow_elmt = WorkflowElementDefinition("ctapipeapplymodels")
    assert isinstance(workflow_elmt.job, CtapipeApplyModelsJob)

    workflow_elmt = WorkflowElementDefinition("unknown")
    assert isinstance(workflow_elmt.job, Job)


def test_we_set_constrained_job_attribute() -> None:
    # Simulation
    workflow_elmt_sim = WorkflowElement(None, "mcsimulation")
    assert getattr(workflow_elmt_sim, "particle", None) is None
    workflow_elmt_sim.set_constrained_job_attribute("particle", "gamma-diffuse")
    assert workflow_elmt_sim.job.particle == "gamma-diffuse"

    # Processing of CORSIKA files with sim_telarray
    workflow_elmt_simtel = WorkflowElement(None, "simtelprocessing")
    assert getattr(workflow_elmt_simtel, "moon", None) is None
    workflow_elmt_simtel.set_constrained_job_attribute("moon", "dark")
    assert workflow_elmt_simtel.job.output_file_metadata["nsb"] == 1

    # Processing
    workflow_elmt_proc = WorkflowElement(
        workflow_elmt_sim.prod_step, "ctapipeprocessing"
    )
    assert getattr(workflow_elmt_proc, "moon", None) is None
    version = "v0.19.2"
    workflow_elmt_proc.set_constrained_job_attribute("version", "v0.19.2")
    assert workflow_elmt_proc.job.version == version


def test_build_input_data() -> None:
    # Simulation
    workflow_elmt_sim = WorkflowElement(None, "mcsimulation")
    assert workflow_elmt_sim.prod_step.Inputquery is None
    workflow_elmt_sim.build_input_data(SIMULATION_CONFIG)
    assert workflow_elmt_sim.prod_step.Inputquery == {}

    # Processing of CORSIKA files with sim_telarray
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_simtel = WorkflowElement(
        workflow_elmt_sim.prod_step, "simtelprocessing"
    )
    assert workflow_elmt_simtel.prod_step.Inputquery is None
    workflow_elmt_simtel.build_input_data(SIM_TELARRAY_PROCESSING_CONFIG)
    expected_simtel_input_query = workflow_elmt_sim.prod_step.Outputquery
    expected_simtel_input_query["nsb"] = 1
    assert workflow_elmt_simtel.prod_step.Inputquery == expected_simtel_input_query

    # Processing
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_proc = WorkflowElement(
        workflow_elmt_sim.prod_step, "ctapipeprocessing"
    )
    assert workflow_elmt_proc.prod_step.Inputquery is None
    workflow_elmt_proc.build_input_data(PROCESSING_CONFIG)
    expected_proc_input_query = workflow_elmt_sim.prod_step.Outputquery
    expected_proc_input_query["nsb"] = 1
    assert workflow_elmt_proc.prod_step.Inputquery == expected_proc_input_query


def test_build_job_attributes() -> None:
    # Simulation
    workflow_elmt_sim = WorkflowElement(None, "mcsimulation")
    workflow_elmt_sim.build_input_data(SIMULATION_CONFIG)
    assert workflow_elmt_sim.job.output_file_metadata == MetadataDict()
    assert workflow_elmt_sim.job.MCCampaign == "ProdTest"

    workflow_elmt_sim.build_job_attributes(WORKFLOW_CONFIG, SIMULATION_CONFIG)
    assert workflow_elmt_sim.job.output_file_metadata == MetadataDict([("nsb", [1])])
    assert workflow_elmt_sim.job.MCCampaign == COMMON_CONFIG["MCCampaign"]

    # Processing of CORSIKA files with sim_telarray
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_simtel = WorkflowElement(
        workflow_elmt_sim.prod_step, "simtelprocessing"
    )
    workflow_elmt_simtel.build_input_data(SIM_TELARRAY_PROCESSING_CONFIG)
    assert workflow_elmt_simtel.job.output_file_metadata == MetadataDict()
    assert workflow_elmt_simtel.job.MCCampaign == "ProdTest"

    workflow_elmt_simtel.build_job_attributes(
        WORKFLOW_CONFIG, SIM_TELARRAY_PROCESSING_CONFIG
    )
    assert workflow_elmt_simtel.job.output_file_metadata == MetadataDict([("nsb", 1)])
    assert workflow_elmt_simtel.job.MCCampaign == COMMON_CONFIG["MCCampaign"]

    # Processing
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_proc = WorkflowElement(
        workflow_elmt_sim.prod_step, "ctapipeprocessing"
    )
    workflow_elmt_proc.build_input_data(PROCESSING_CONFIG)
    assert workflow_elmt_proc.job.output_file_metadata == MetadataDict()
    assert workflow_elmt_proc.job.MCCampaign == "ProdTest"

    workflow_elmt_proc.build_job_attributes(WORKFLOW_CONFIG, PROCESSING_CONFIG)
    assert workflow_elmt_proc.job.output_file_metadata == MetadataDict([("nsb", 1)])
    assert workflow_elmt_proc.job.MCCampaign == COMMON_CONFIG["MCCampaign"]


def test_build_job_output_data() -> None:
    # Simulation
    workflow_elmt_sim = WorkflowElement(None, "mcsimulation")
    workflow_elmt_sim.build_input_data(SIMULATION_CONFIG)
    workflow_elmt_sim.build_job_attributes(WORKFLOW_CONFIG, SIMULATION_CONFIG)
    assert workflow_elmt_sim.job.output_metadata == MetadataDict()
    workflow_elmt_sim.build_job_output_data(SIMULATION_CONFIG)
    assert workflow_elmt_sim.job.output_metadata == SIMULATION_OUTPUT_METADATA

    # Processing of CORSIKA files with sim_telarray
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_simtel = WorkflowElement(
        workflow_elmt_sim.prod_step, "simtelprocessing"
    )
    workflow_elmt_simtel.build_input_data(SIM_TELARRAY_PROCESSING_CONFIG)
    workflow_elmt_simtel.build_job_attributes(
        WORKFLOW_CONFIG, SIM_TELARRAY_PROCESSING_CONFIG
    )
    assert workflow_elmt_simtel.job.output_metadata == MetadataDict()
    workflow_elmt_simtel.build_job_output_data(SIM_TELARRAY_PROCESSING_CONFIG)
    assert workflow_elmt_simtel.job.output_metadata == SIMULATION_OUTPUT_METADATA

    # Processing
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_proc = WorkflowElement(
        workflow_elmt_sim.prod_step, "ctapipeprocessing"
    )
    workflow_elmt_proc.build_input_data(PROCESSING_CONFIG)
    workflow_elmt_proc.build_job_attributes(WORKFLOW_CONFIG, PROCESSING_CONFIG)
    assert workflow_elmt_proc.job.output_metadata == MetadataDict()
    workflow_elmt_proc.build_job_output_data(PROCESSING_CONFIG)
    assert workflow_elmt_proc.job.output_metadata == CTAPIPE_PROCESS_OUTPUT_METADATA


def test_build_element_config() -> None:
    workflow_elmt_sim = WorkflowElement(None, "mcsimulation")
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_proc = WorkflowElement(
        workflow_elmt_sim.prod_step, "ctapipeprocessing"
    )
    workflow_elmt_proc.prod_step.Outputquery = deepcopy(CTAPIPE_PROCESS_OUTPUT_METADATA)
    worflow_elmt_merge = WorkflowElement(workflow_elmt_proc.prod_step, "merging")
    worflow_elmt_merge.build_input_data(MERGING_CONFIG_1)
    worflow_elmt_merge.build_job_attributes(WORKFLOW_CONFIG, MERGING_CONFIG_1)
    worflow_elmt_merge.build_job_output_data(MERGING_CONFIG_1)
    worflow_elmt_merge.build_element_config(MERGING_CONFIG_1)
    assert worflow_elmt_merge.prod_step.GroupSize == 5
    # Get "descr_short" and remove empty ones
    descr_short: list[str] = [
        line.replace("<descr_short>", "").replace("</descr_short>", "")
        for line in worflow_elmt_merge.prod_step.Body.split("\n")
        if "descr_short" in line
        and line.replace("<descr_short>", "").replace("</descr_short>", "")
    ]
    expected_desc = {
        "Setup software",
        "Run ctapipe merge",
        "Save data files to SE and register them in DFC",
        "Tag files as unused if job failed",
    }
    assert set(descr_short).issubset(expected_desc)


def test_get_merging_level() -> None:
    workflow_elmt_sim = WorkflowElement(None, "mcsimulation")
    workflow_elmt_sim.prod_step.Outputquery = deepcopy(SIMULATION_OUTPUT_METADATA)
    workflow_elmt_proc = WorkflowElement(
        workflow_elmt_sim.prod_step, "ctapipeprocessing"
    )
    workflow_elmt_proc.prod_step.Outputquery = deepcopy(CTAPIPE_PROCESS_OUTPUT_METADATA)
    worflow_elmt_merge = WorkflowElement(workflow_elmt_proc.prod_step, "merging")
    assert worflow_elmt_merge.job.merged == 0
    res_merged = worflow_elmt_merge.get_merging_level()
    assert res_merged == 0
    worflow_elmt_merge.prod_step.ParentStep.Outputquery = deepcopy(
        CTAPIPE_PROCESS_OUTPUT_METADATA
    )
    worflow_elmt_merge.prod_step.ParentStep.Outputquery["merged"] = 6
    res_merged = worflow_elmt_merge.get_merging_level()
    assert res_merged == 6



#!/usr/bin/env python
""" General script to setup software through the SoftwareManager module
        J. Bregeon, L. Arrabito 15/09/2019
"""

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC.Core.Base.Script import Script

Script.registerSwitch("p:", "Package=", "Software package name")
Script.registerSwitch("v:", "Version=", "Base version to look for")
Script.registerSwitch("a:", "Category=", "Program category (simulations, analysis...)")
Script.registerSwitch("g:", "Compiler=", "Target a compiler_optimization configuration")
Script.registerSwitch("r:", "Repository=", "Target CVMFS repository")

Script.setUsageMessage(
    "\n".join(
        [
            __doc__.split("\n")[1],
            "Usage:",
            "  %s -p package -v version -a [program_category] -g [compiler]"
            % Script.scriptName,
            "Arguments:",
            "  package: corsika_simtelarray",
            "  version: 2019-09-03",
            "  program_category: simulations",
            "  compiler: gcc48_avx2",
            f"\ne.g: {Script.scriptName} -p corsika_simtelarray -v 2019-09-03",
        ]
    )
)

Script.parseCommandLine(ignoreErrors=False)


# Specific DIRAC imports
from CTADIRAC.Core.Utilities.SoftwareManager import SoftwareManager


@Script()
def main():
    """setup a given software package to be used in the main workflow

    Keyword arguments:
        package: software name
        version: software version
        category: simulations, analysis...
        compiler: compiler and configuration
    """
    package = None
    version = None
    category = "simulations"
    compiler = "gcc48_default"
    repository = None
    for switch in Script.getUnprocessedSwitches():
        if switch[0] == "p" or switch[0].lower() == "package":
            package = switch[1]
        if switch[0] == "v" or switch[0].lower() == "version":
            version = switch[1]
        if switch[0] == "a" or switch[0].lower() == "category":
            category = switch[1]
        if switch[0] == "g" or switch[0].lower() == "compiler":
            compiler = switch[1]
        if switch[0] == "r" or switch[0].lower() == "repository":
            repository = switch[1]
    if package is None or version is None:
        DIRAC.gLogger.error("Please give a package name and a version")
        DIRAC.exit(-1)
    DIRAC.gLogger.notice(f"Trying to setup: {package} {version} {category} {compiler}")
    # get arguments
    soft_category = {package: category}
    manager = SoftwareManager(soft_category)
    # for testing only
    if repository is not None:
        manager.CVMFS_DIR = repository
    # check if and where Package is available
    # return cvmfs/tarball and full path
    res = manager.find_software(package, version, compiler)
    if not res["OK"]:  # could not find package
        return res
    source = res["Value"]["Source"]
    package_dir = res["Value"]["Path"]
    if source == "cvmfs":
        res = manager.install_dirac_scripts(package_dir)
        if not res["OK"]:
            return res
        res = manager.dump_setup_script_path(package_dir)
        if not res["OK"]:
            return res

    DIRAC.gLogger.notice("Setup software completed successfully")
    DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
import os
from multiprocessing import Pool

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Core.Base.Script import Script

Script.setUsageMessage(
    """
Bulk upload of a list of local files from the current directory to a Storage Element
Usage:
   cta-prod-add-file <ascii file with lfn list> <SE>
"""
)

Script.parseCommandLine(ignoreErrors=True)

args = Script.getPositionalArgs()
if len(args) > 1:
    infile = args[0]
    SE = args[1]
else:
    Script.showHelp()

# Import of Dirac must comes after parseCommandLine
from DIRAC.Interfaces.API.Dirac import Dirac


@Script()
def main():
    infileList = read_inputs_from_file(infile)
    p = Pool(10)
    p.map(addfile, infileList)


def addfile(lfn):
    dirac = Dirac()
    res = dirac.addFile(lfn, os.path.basename(lfn), SE)
    if not res["OK"]:
        gLogger.error("Error uploading file", lfn)
        return res["Message"]


if __name__ == "__main__":
    main()



""" Class that contains client access to the Provenance DB handler. """

__RCSID__ = "$Id$"

# # from DIRAC
from DIRAC.Core.Base.Client import Client


class ProvClient(Client):

    """Exposes the functionality available in the DataManagementSystem/ProvenanceManagerHandler"""

    def __init__(self, url=None, **kwargs):
        """Simple constructor"""

        Client.__init__(self, **kwargs)
        res = self.serverURL = "DataManagement/ProvenanceManager" if not url else url

    def addActivity(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addActivity(rowJSON)

    def addDatasetEntity(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addDatasetEntity(rowJSON)

    def addValueEntity(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addValueEntity(rowJSON)

    def addUsed(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addUsed(rowJSON)

    def addWasGeneratedBy(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addWasGeneratedBy(rowJSON)

    def addAgent(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addAgent(rowJSON)

    def addWasAttributedTo(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addWasAttributedTo(rowJSON)

    def addWasAssociatedWith(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addWasAssociatedWith(rowJSON)

    def addActivityDescription(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addActivityDescription(rowJSON)

    def addDatasetDescription(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addDatasetDescription(rowJSON)

    def addUsageDescription(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addUsageDescription(rowJSON)

    def addGenerationDescription(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addGenerationDescription(rowJSON)

    def addValueDescription(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addValueDescription(rowJSON)

    def addWasConfiguredBy(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addWasConfiguredBy(rowJSON)

    def addParameter(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addParameter(rowJSON)

    def addConfigFile(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addConfigFile(rowJSON)

    def addParameterDescription(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addParameterDescription(rowJSON)

    def addConfigFileDescription(self, row):
        res = row.toJSON()
        rowJSON = res["Value"]
        rpcClient = self._getRPC()
        return rpcClient.addConfigFileDescription(rowJSON)

    def getAgents(self):
        rpcClient = self._getRPC()
        return rpcClient.getAgents()

    def getAgentKey(self, agent_id):
        rpcClient = self._getRPC()
        return rpcClient.getAgentKey(agent_id)

    def getUsageDescription(self, activityDescription_id, role):
        rpcClient = self._getRPC()
        return rpcClient.getUsageDescription(activityDescription_id, role)

    def getParameterDescription(self, activityDescription_id, parameter_name):
        rpcClient = self._getRPC()
        return rpcClient.getParameterDescription(activityDescription_id, parameter_name)

    def getConfigFileDescription(self, activityDescription_id, configFile_name):
        rpcClient = self._getRPC()
        return rpcClient.getConfigFileDescription(
            activityDescription_id, configFile_name
        )

    def getActvityDescriptionKey(
        self, activityDescription_name, activityDescription_version
    ):
        rpcClient = self._getRPC()
        return rpcClient.getActivityDescriptionKey(
            activityDescription_name, activityDescription_version
        )



#!/bin/env python
"""
  Simple terminal job error summary
"""

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from CTADIRAC.Core.Utilities import tool_box


@Script()
def main():
    """Select jobs based on conditions"""
    Script.registerSwitch("", "owner=", "the job owner")
    Script.registerSwitch("", "jobGroup=", "the job group")
    Script.registerSwitch("", "hours=", "Get status for jobs of the last n hours")
    Script.registerSwitch(
        "", "failed=", '1 or 0 : Save or not failed jobs in "failed.txt"'
    )
    switches, argss = Script.parseCommandLine(ignoreErrors=True)

    # defaults
    owner = "arrabito"
    job_group = ""
    n_hours = 24
    save_failed = False

    for switch in switches:
        if switch[0].lower() == "owner":
            owner = switch[1]
        elif switch[0].lower() == "jobgroup":
            job_group = switch[1]
        elif switch[0].lower() == "hours":
            n_hours = int(switch[1])
        elif switch[0].lower() == "failed":
            save_failed = int(switch[1])

    # Start doing something
    # import Dirac here after parseCommandLine
    from DIRAC.Interfaces.API.Dirac import Dirac

    dirac = Dirac()

    # do the jobs via the 2 main methods
    jobs_list = tool_box.get_job_list(owner, job_group, n_hours)
    gLogger.notice(
        '%s jobs found for group "%s" and owner "%s" in the past %s hours\n'
        % (len(jobs_list), job_group, owner, n_hours)
    )

    # get jobs status
    status = dirac.getJobStatus(jobs_list)

    # print out my favourite tables
    SitesDict = {}

    for job in jobs_list:
        #    print job, status['Value'][int(job)]
        site = status["Value"][int(job)]["Site"]
        #    site=status['Value'][int(job)]['CE']
        minstatus = status["Value"][int(job)]["MinorStatus"]
        majstatus = status["Value"][int(job)]["Status"]

        if majstatus not in {"Done", "Failed"}:
            continue

        if site not in SitesDict.keys():
            if site.find(".") == -1:
                site = "    None"  # note that blank spaces are needed
            SitesDict[site] = {"Total": 0, "Failed": 0, "Errors": {}}

        SitesDict[site]["Total"] += 1
        if majstatus == "Failed":
            SitesDict[site]["Failed"] += 1
            if minstatus not in SitesDict[site]["Errors"].keys():
                SitesDict[site]["Errors"][minstatus] = 0
            SitesDict[site]["Errors"][minstatus] += 1

    gLogger.notice("%20s  Finish  Errors  Rate  Failure reason" % "Site")
    for site, val in sorted(SitesDict.items()):
        errstr = ""
        for error, amount in val["Errors"].items():
            if len(errstr) > 0:
                errstr += "\n\t\t\t\t\t    "
            errstr += "%s (%d)" % (error, amount)

        txt = "%20s%8d%8d%5d%%  %s" % (
            site,
            val["Total"],
            val["Failed"],
            val["Failed"] * 100 / val["Total"],
            errstr,
        )
        gLogger.notice(txt)

    # saved information on failed jobs on disk
    if save_failed:
        txt = ""
        for job in jobs_list:
            majstatus = status["Value"][int(job)]["Status"]
            if majstatus == "Failed":
                txt += str(dirac.getJobSummary(int(job))) + "\n"
        open("failed.txt", "w").write(txt)
        gLogger.notice("Saved failed.txt on disk")

    DIRAC.exit()


if __name__ == "__main__":
    main()
